#!/usr/bin/env python3

import sys
import os
import unittest

sys.path.insert(0, os.path.dirname(os.path.realpath(__file__)) + "/../build")

from pywpsrpc.rpcetapi import etapi
from pywpsrpc import (rpcetapi, common)
from pywpsrpc.utils import RpcIter, RpcProxy


class TestRpcEtApi(unittest.TestCase):

    def setUp(self):
        super().setUp()
        _, self.rpc = rpcetapi.createEtRpcInstance()
        _, self.app = self.rpc.getEtApplication()

    def tearDown(self):
        self.app.Quit()
        super().tearDown()

    def check_call(self, funcName, hr, value=None):
        self.assertEqual(hr, common.S_OK, funcName)

    def test(self):
        hr, pid = self.rpc.getProcessPid()
        self.check_call("rpc.getProcessPid", hr, pid)

        hr, workbooks = self.app.get_Workbooks()
        self.check_call("app.get_Workbooks", hr, workbooks)

        hr, workbook = workbooks.Add()
        self.check_call("workbooks.Add", hr, workbook)

        hr, sheet = workbook.get_ActiveSheet()
        self.check_call("workbook.get_ActiveSheet", hr, sheet)

        hr, rg = sheet.get_Range("A1")
        self.check_call("sheet.get_Range", hr, rg)

        hr = rg.put_Value(RHS="test")
        self.check_call("rg.put_Value", hr)

        rg = sheet.Range("A1")
        self.assertEqual(rg.Value, "test")

        hr, window = self.app.get_ActiveWindow()
        self.check_call("app.get_ActiveWindow", hr, window)

        hr, winType = window.get_Type()
        self.check_call("window.get_Type", hr, winType)

        props = workbook.CustomDocumentProperties
        for prop in RpcIter(props):
            self.assertIsNotNone(prop.Name)

        props = workbook.BuiltinDocumentProperties
        for prop in RpcIter(props):
            self.assertIsNotNone(prop.Name)

        hr = workbook.Close(False)
        self.check_call("workbook.Close", hr)

    def test_pagesetup(self):
        _, workbook = self.app.Workbooks.Add()
        ps = workbook.ActiveSheet.PageSetup
        self.assertEqual(ps.Pages.Count, 0)

        ps.FitToPagesTall = True
        self.assertTrue(ps.FitToPagesTall)

        ps.FitToPagesWide = False
        self.assertFalse(ps.FitToPagesWide)

        workbook.Close(False)

    def test_querytable(self):
        _, workbook = self.app.Workbooks.Add()
        sheet = workbook.ActiveSheet
        self.assertEqual(sheet.QueryTables.Count, 0)
        workbook.Close(False)

    def test_formatcondition(self):
        _, workbook = self.app.Workbooks.Add()
        sheet = workbook.ActiveSheet
        rg = sheet.Range("A1")

        fc = rg.FormatConditions
        hr, cond = fc.AddAboveAverage()

        self.assertEqual(fc.Count, 1)
        self.assertIsNotNone(fc[1])

        cond.AboveBelow = etapi.xlAboveAverage
        self.assertEqual(cond.AboveBelow, etapi.xlAboveAverage)

        workbook.Close(False)

    def test_charts(self):
        _, workbook = self.app.Workbooks.Add()
        self.assertEqual(workbook.Charts.Count, 0)
        workbook.Close(False)

    def test_autofill(self):
        _, workbook = self.app.Workbooks.Add()
        sheet = workbook.ActiveSheet
        rg = sheet.Range("A1")
        rg.put_Value(RHS="1")

        rg = sheet.Range("A2")
        rg.put_Value(RHS="2")

        sourceRg = sheet.Range("A1:A2")
        fillRg = sheet.Range("A1:A20")
        hr, _ = sourceRg.AutoFill(fillRg)
        self.assertEqual(hr, common.S_OK)
        workbook.Close(False)

    def test_shapes(self):
        workbook = RpcProxy(self.app.Workbooks.Add())
        shapes = workbook.ActiveSheet.Shapes
        line = shapes.AddLine(10, 10, 250, 250).Line
        self.assertNotEqual(line, None)
        self.assertEqual(1, shapes.Count)

        line.DashStyle = etapi.msoLineDashDotDot

        for shape in shapes:
            self.assertEqual(shape.Type, etapi.msoLine)
            self.assertEqual(shape.Line.DashStyle, etapi.msoLineDashDotDot)

        workbook.Close(False)

    def test_array(self):
        _, workbook = self.app.Workbooks.Add()
        sheet = workbook.ActiveSheet
        sheet.Range("A1:B1").Value = [1, "a"]
        self.assertEqual(sheet.Range("A1").Value, 1)
        self.assertEqual(sheet.Range("B1").Value, "a")

        value = sheet.Range("A1:B1").Value
        self.assertTrue(isinstance(value, list))
        self.assertTrue(len(value) == 1)
        self.assertEqual(value[0][0], 1)
        self.assertEqual(value[0][1], "a")

        # FIXME: maybe et doesn't support 2-d array???
        sheet.Range("A2:B3").Value = [[2, 3], [4, 5]]
        value = sheet.Range("A2:B3").Value
        self.assertEqual(len(value), 2)
        #self.assertEqual(value[1][1], 5)

        workbook.Close(False)


if __name__ == "__main__":
    unittest.main()



#**
# * Copyright (c) 2020 Weitian Leung
# *
# * This file is part of pywpsrpc.
# *
# * This file is distributed under the MIT License.
# * See the LICENSE file for details.
# *
#*

from .utils import (RpcException, RpcProxy, RpcIter)



#!/usr/bin/python3

#**
# * Copyright (c) 2020 Weitian Leung
# *
# * This file is part of pywpsrpc.
# *
# * This file is distributed under the MIT License.
# * See the LICENSE file for details.
# *
#*


import sys

from pywpsrpc.rpcwpsapi import (createWpsRpcInstance, wpsapi)
from pywpsrpc.common import FAILED

from PySide2.QtWidgets import *
from PySide2.QtCore import *
from PySide2.QtGui import QWindow


class WpsWindow(QWidget):

    def __init__(self, parent=None):
        super().__init__(parent)

        self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        self.setAttribute(Qt.WA_NativeWindow, True)

        layout = QVBoxLayout(self)

        self._container = None

    def setContainer(self, container):
        if self._container:
            self.layout().removeWidget(self._container)
            self._container = None

        self.layout().addWidget(container)
        self._container = container


class MyWindow(QWidget):

    def __init__(self, parent=None):
        super().__init__(parent)

        self._wpsApp = None

        layout = QVBoxLayout(self)

        hbox = QHBoxLayout()
        layout.addLayout(hbox)

        btnOpen = QPushButton("Open Wps", self)
        btnOpen.clicked.connect(self._onOpenWps)
        hbox.addWidget(btnOpen)

        self.btnOpenDoc = QPushButton("Open Document", self)
        self.btnOpenDoc.clicked.connect(self._onOpenDocument)
        self.btnOpenDoc.setEnabled(False)
        hbox.addWidget(self.btnOpenDoc)

        hbox.addStretch()

        self._wpsWnd = WpsWindow(self)
        layout.addWidget(self._wpsWnd)

        hbox = QHBoxLayout()
        hbox.addWidget(QLabel(self.tr("Current Document: "), self))
        self.lbCurDocName = QLabel(self.tr("<No document>"), self)
        hbox.addWidget(self.lbCurDocName)
        hbox.addStretch()
        layout.addLayout(hbox)

        hbox = QHBoxLayout()
        hbox.addWidget(QLabel(self.tr("Current Selection: "), self))
        self.lbCurSelection = QLabel(self.tr("<No selection>"), self)
        hbox.addWidget(self.lbCurSelection)
        hbox.addStretch()
        layout.addLayout(hbox)

    def _onOpenWps(self):
        if self._wpsApp:
            return

        hr, rpc = createWpsRpcInstance()
        if FAILED(hr):
            QMessageBox.critical(
                self, "Open Wps", "Failed to call createWpsRpcInstance")
            return

        args = ["-shield", "-multiply", "-x11embed",
                "{}".format(self._wpsWnd.winId()),
                "{}".format(self._wpsWnd.width()),
                "{}".format(self._wpsWnd.height())
                ]

        rpc.setProcessArgs(args)

        hr, self._wpsApp = rpc.getWpsApplication()
        if not self._wpsApp:
            QMessageBox.critical(
                self, "Open Wps", "Failed to call getWpsApplication")
            return

        hr = rpc.registerEvent(self._wpsApp,
                               wpsapi.DIID_ApplicationEvents4,
                               "WindowBeforeRightClick",
                               self._onWindowBeforeRightClick)

        hr = rpc.registerEvent(self._wpsApp,
                               wpsapi.DIID_ApplicationEvents4,
                               "WindowSelectionChange",
                               self._onWindowSelectionChange)

        hr = rpc.registerEvent(self._wpsApp,
                               wpsapi.DIID_ApplicationEvents4,
                               "DocumentChange",
                               self._onDocumentChange)

        hr = rpc.registerEvent(self._wpsApp,
                               wpsapi.DIID_ApplicationEvents4,
                               "DocumentOpen",
                               self._onDocumentOpen)

        hr = rpc.registerEvent(self._wpsApp,
                               wpsapi.DIID_ApplicationEvents4,
                               "NewDocument",
                               self._onNewDocument)
        appEx = self._wpsApp.ApplicationEx
        if appEx:
            container = QWidget.createWindowContainer(
                QWindow.fromWinId(appEx.EmbedWid), self)
            self._wpsWnd.setContainer(container)
            # FIXME: the container isn't place correctly
            if not self.isMaximized():
                self.resize(self.width() + 1, self.height())

        self.btnOpenDoc.setEnabled(True)

    def _onOpenDocument(self):
        if not self._wpsApp:
            return

        filePath, _ = QFileDialog.getOpenFileName(
            self,
            "Select File",
            QStandardPaths.standardLocations(
                QStandardPaths.DocumentsLocation)[0]
        )

        if not filePath:
            return

        hr, _ = self._wpsApp.Documents.Open(filePath)
        if FAILED(hr):
            QMessageBox.critical(
                self, "Demo", "Failed to call open document '%s'" % filePath)

    def _onWindowBeforeRightClick(self, selection):
        btn = QMessageBox.question(
            self,
            self.tr("About to show popup menu"),
            self.tr('Press "No" to cancel the popup menu.'))

        return btn == QMessageBox.No

    def _onWindowSelectionChange(self, selection):
        start = selection.Start
        end = selection.End
        if start == end:
            self.lbCurSelection.setText(self.tr("<No selection>"))
        else:
            text = selection.Text
            if text:
                fm = self.lbCurDocName.fontMetrics()
                text = fm.elidedText(text, Qt.ElideRight, self.width() / 2)
                self.lbCurSelection.setText(text)
            else:
                self.lbCurSelection.setText(self.tr("<No text>"))

    def _onDocumentChange(self):
        text = self.tr("<No document>")
        if self._wpsApp:
            docs = self._wpsApp.Documents
            if docs.Count > 0:
                doc = self._wpsApp.ActiveDocument
                text = doc.Name

        self.lbCurDocName.setText(text)

    def _onDocumentOpen(self, doc):
        self._onDocumentChange()

    def _onNewDocument(self, doc):
        self._onDocumentChange()

    def closeEvent(self, event):
        if self._wpsApp:
            self._wpsApp.Quit()
        event.accept()


def main():
    app = QApplication(sys.argv)

    window = MyWindow()
    window.resize(800, 600)
    window.show()

    sys.exit(app.exec_())


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

import sys
import os
import time
import unittest

sys.path.insert(0, os.path.dirname(os.path.realpath(__file__)) + "/../build")
from pywpsrpc import rpcwpsapi
from pywpsrpc import common
from pywpsrpc.rpcwpsapi import wpsapi
from pywpsrpc.utils import RpcIter

class TestRpcWpsApi(unittest.TestCase):

    def check_call(self, funcName, hr):
        self.assertEqual(hr, common.S_OK, funcName)

    def test(self):
        hr, rpc = rpcwpsapi.createWpsRpcInstance()
        self.check_call("createWpsRpcInstance", hr)

        hr = rpc.setProcessArgs(
            [os.path.dirname(os.path.realpath(__file__)) + "/../pyproject.toml"])
        self.check_call("setProcessArgs", hr)

        hr, wpsApp = rpc.getWpsApplication()
        self.check_call("rpc.getWpsApplication", hr)

        hr, pid = rpc.getProcessPid()
        self.check_call("getProcessPid", hr)
        self.assertNotEqual(pid, 0, "rpc.getProcessPid")

        hr, docs = wpsApp.get_Documents()
        self.check_call("get_Documents", hr)

        # FIXME: some env may fails get_Count (returns 0)
        time.sleep(0.5)

        hr, count = docs.get_Count()
        self.check_call("get_Count", hr)
        self.assertEqual(count, 1)

        hr, doc = wpsApp.get_ActiveDocument()
        self.check_call("get_ActiveDocument", hr)

        hr, name = doc.get_Name()
        self.check_call("get_Name", hr)
        self.assertEqual(name, "pyproject.toml", "doc.get_Name")

        hr, docRange = doc.get_Content()
        self.check_call("get_Content", hr)

        hr, start = docRange.get_Start()
        self.check_call("get_Start", hr)

        hr, oldEnd = docRange.get_End()
        self.check_call("get_End", hr)

        hr = docRange.InsertParagraphAfter()
        self.check_call("InsertParagraphAfter", hr)

        hr, end = docRange.get_End()
        self.assertEqual(end, oldEnd + 1, "InsertParagraphAfter")

        hr, text = docRange.get_Text()
        self.check_call("get_Text", hr)

        hr = docRange.InsertAfter("Hello, world")
        self.check_call("InsertAfter", hr)

        end -= 1
        hr = docRange.put_Start(end)
        self.check_call("put_Start", hr)

        hr = docRange.put_End(end + 12)
        self.check_call("put_End", hr)

        hr, text = docRange.get_Text()
        self.check_call("get_Text", hr)
        self.assertEqual(text, "Hello, world", "docRange.get_Text")

        hr, paras = doc.get_Paragraphs()
        self.check_call("get_Paragraphs", hr)

        hr, count = paras.get_Count()
        self.check_call("get_Count", hr)

        docRange.InsertParagraphAfter()

        hr, newCount = paras.get_Count()
        self.check_call("get_Count", hr)
        self.assertEqual(newCount, count + 1, "paras.get_Count")

        hr, app2 = paras.get_Application()
        self.check_call("paras.get_Application", hr)

        hr, build = app2.get_Build()
        self.check_call("app2.get_Build", hr)

        hr, para = paras.get_First()
        self.check_call("paras.get_First", hr)

        hr, dropCap = para.get_DropCap()
        self.check_call("para.get_DropCap", hr)

        hr = dropCap.put_Position(wpsapi.wdDropNormal)
        self.check_call("dropCap.put_Position", hr)

        hr, paraRange = para.get_Range()
        self.check_call("para.get_Range", hr)

        hr, text = paraRange.get_Text()
        self.check_call("paraRange.get_Text", hr)

        hr = para.put_Alignment(wpsapi.wdAlignParagraphCenter)
        self.check_call("para.put_Alignment", hr)

        hr, paraFormat = para.get_Format()
        self.check_call("para.get_Format", hr)

        hr, style = paraFormat.get_Style()
        self.check_call("paraFormat.get_Style", hr)

        hr, borders = paraFormat.get_Borders()
        self.check_call("paraFormat.get_Borders", hr)

        hr, count = borders.get_Count()
        self.check_call("borders.get_Count", hr)

        hr = borders.put_Shadow(True)
        self.check_call("borders.put_Shadow", hr)

        hr = paraFormat.put_Borders(borders)
        self.check_call("paraFormat.put_Borders", hr)

        hr, shading = para.get_Shading()
        self.check_call("para.get_Shading", hr)

        hr = shading.put_BackgroundPatternColor(wpsapi.wdColorGreen)
        self.check_call("shading.put_BackgroundPatternColor", hr)

        hr, tabStops = para.get_TabStops()
        self.check_call("para.get_TabStops", hr)

        hr, count = tabStops.get_Count()
        self.check_call("tabStops.get_Count", hr)

        # reset the range to find
        hr, docRange = doc.get_Content()

        hr, find = docRange.get_Find()
        self.check_call("docRange.get_Find", hr)

        hr, found = find.Execute("Hello, world", MatchWholeWord=True)
        self.check_call("find.Execute", hr)
        self.assertTrue(found, "find.Execute")

        hr, font = docRange.get_Font()
        self.check_call("docRange.get_Font", hr)

        hr, name = font.get_Name()
        self.check_call("font.get_Name", hr)

        hr = font.put_Bold(True)
        self.check_call("font.put_Bold", hr)

        hr, selection = wpsApp.get_Selection()
        self.check_call("get_Selection", hr)

        hr = selection.SetRange(0, 10)
        self.check_call("SetRange", hr)

        hr, start = selection.get_Start()
        self.check_call("get_Start", hr)

        hr, end = selection.get_End()
        self.check_call("get_End", hr)

        hr, text = selection.get_Text()
        self.check_call("get_Text", hr)

        hr, tables = doc.get_Tables()
        self.check_call("doc.get_Tables", hr)

        hr, pos = selection.EndKey(Unit=wpsapi.wdStory)
        self.check_call("selection.EndKey", hr)

        hr = selection.InsertParagraphAfter()
        hr, pos = selection.EndKey(Unit=wpsapi.wdStory)

        hr, selRange = selection.get_Range()
        self.check_call("selection.get_Range", hr)

        hr, table = tables.Add(selRange, 2, 2)
        self.check_call("tables.Add", hr)

        hr, cell = table.Cell(1, 2)
        self.check_call("table.Cell", hr)

        hr, cellRange = cell.get_Range()
        self.check_call("cell.get_Range", hr)

        hr = cellRange.put_Text("cell")
        self.check_call("cellRange.put_Text", hr)

        hr, shapes = doc.get_Shapes()
        self.check_call("doc.get_Shapes", hr)

        hr, shape = shapes.AddShape(wpsapi.msoShapeMoon, 0, 0, 100, 100)
        self.check_call("shapes.AddShape", hr)

        hr = shape.put_AlternativeText("this is a moon shape")
        self.check_call("shape.put_AlternativeText", hr)

        hr, fields = doc.get_Fields()
        self.check_call("doc.get_Fields", hr)

        hr, field = fields.Add(selRange, Type=wpsapi.wdFieldDate)
        self.check_call("fields.Add", hr)

        hr, bookmarks = doc.get_Bookmarks()
        self.check_call("doc.get_Bookmarks", hr)

        hr, bkmRange = doc.Range(0, 10)
        self.check_call("doc.Range", hr)

        hr, bookmark = bookmarks.Add("BookmarkName")
        self.check_call("bookmarks.Add", hr)

        hr, count = bookmarks.get_Count()
        self.check_call("bookmarks.get_Count", hr)

        hr, name = bookmark.get_Name()
        self.check_call("bookmark.get_Name", hr)

        hr, tocs = doc.get_TablesOfContents()
        self.check_call("doc.get_TablesOfContents", hr)

        hr, bkmRange = bookmark.get_Range()
        self.check_call("bookmark.get_Range", hr)

        hr, toc = tocs.Add(bkmRange)
        self.check_call("tocs.Add", hr)

        hr = doc.SaveAs("test.docx", wpsapi.wdFormatDocument)
        self.check_call("SaveAs", hr)

        hr = doc.Close()
        self.check_call("Close", hr)

        hr, doc = docs.Open("test.docx", ReadOnly=True)
        self.check_call("Open", hr)

        hr, readOnly = doc.get_ReadOnly()
        self.check_call("get_ReadOnly", hr)
        self.assertTrue(readOnly, "doc.get_ReadOnly")

        hr, window = doc.get_ActiveWindow()
        self.check_call("doc.get_ActiveWindow", hr)

        hr, caption = window.get_Caption()
        self.check_call("window.get_Caption", hr)

        doc.Close()
        self.check_call("Close", hr)

        hr, doc = docs.Add(Visible=False)
        self.check_call("Add", hr)

        hr, name = doc.get_Name()
        self.check_call("get_Name", hr)

        props = doc.CustomDocumentProperties
        for prop in RpcIter(props):
            self.assertIsNotNone(prop.Name)

        props = doc.BuiltInDocumentProperties
        for prop in RpcIter(props):
            self.assertIsNotNone(prop.Name)

        hr, name = wpsApp.get_Name()
        self.check_call("get_Name", hr)

        hr, version = wpsApp.get_Version()
        self.check_call("get_Version", hr)

        hr, build = wpsApp.get_Build()
        self.check_call("get_Build", hr)

        hr, caption = wpsApp.get_Caption()
        self.check_call("get_Caption", hr)

        hr = wpsApp.put_Caption("test标题")
        self.check_call("put_Caption", hr)

        hr, caption = wpsApp.get_Caption()
        self.check_call("get_Caption", hr)
        self.assertEqual(caption, "test标题")

        hr = wpsApp.put_Visible(False)
        self.check_call("put_Visible", hr)

        hr, visible = wpsApp.get_Visible()
        self.check_call("get_Visible", hr)

        hr = wpsApp.put_Visible(True)
        self.check_call("put_Visible", hr)

        hr, visible = wpsApp.get_Visible()
        self.check_call("get_Visible", hr)
        self.assertTrue(visible, "wpsApp.get_Visible")

        if not wpsApp.openDocument(os.path.dirname(os.path.realpath(__file__)) + "/../pyproject.toml", True):
            print("Can't open document")

        self.assertTrue(wpsApp.insertDocumentField("test"))
        self.assertTrue(wpsApp.insertDocumentField("公文域"))
        self.assertEqual(wpsApp.getAllDocumentField(), "test,公文域")
        # wpsApp.getForceBackUpEnabled()

        hr = wpsApp.Quit(wpsapi.wdDoNotSaveChanges)
        self.check_call("Quit", hr)

    def test_cmp_docs(self):
        _, rpc = rpcwpsapi.createWpsRpcInstance()
        _, app = rpc.getWpsApplication()

        _, doc1 = app.Documents.Add()
        _, doc2 = app.Documents.Add()
        doc2.Content.InsertAfter("test")

        _, doc = app.CompareDocuments(doc1, doc2)
        # FIXME: seems not works for linux..
        # self.assertNotEqual(doc, None)

        app.Quit(wpsapi.wdDoNotSaveChanges)


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3

import sys
import os
import unittest

sys.path.insert(0, os.path.dirname(os.path.realpath(__file__)) + "/../build")
from pywpsrpc import RpcProxy, RpcIter
from pywpsrpc.rpcwpsapi import createWpsRpcInstance
from pywpsrpc.rpcwppapi import createWppRpcInstance
from pywpsrpc.rpcetapi import createEtRpcInstance


class TestWps(unittest.TestCase):

    def test_wps(self):
        hr, rpc = createWpsRpcInstance()
        app = RpcProxy(rpc.getWpsApplication())
        docs = app.Documents

        docs.Add()
        self.assertEqual(len(docs), 1)

        docs.Add()
        self.assertEqual(len(docs), 2)

        # index starts from 1, not 0
        for i in range(1, len(docs) + 1):
            self.assertIsNotNone(docs[i].Name)

        for doc in docs:
            self.assertIsNotNone(doc.Name)

        # use raw object
        for doc in RpcIter(docs.rpc_object):
           self.assertIsNotNone(doc.Name)

        # access by name
        name = docs[2].Name
        doc = docs[name]
        self.assertEqual(name, doc.Name)

        doc = docs.Open(os.path.dirname(os.path.realpath(__file__)) + "/../pyproject.toml", ReadOnly=True)
        for para in doc.Paragraphs:
            self.assertIsNotNone(para.Range.Text)

        app.Quit()


class TestWpp(unittest.TestCase):

    def test_wpp(self):
        hr, rpc = createWppRpcInstance()
        app = RpcProxy(rpc.getWppApplication())

        preses = app.Presentations
        self.assertEqual(len(preses), 0)

        preses.Add()
        self.assertEqual(len(preses), 1)

        preses.Add()
        self.assertEqual(len(preses), 2)

        for pres in preses:
            self.assertIsNotNone(pres.Name)

        app.Quit()


class TestEt(unittest.TestCase):

    def test_et(self):
        hr, rpc = createEtRpcInstance()
        app = RpcProxy(rpc.getEtApplication())

        wbs = app.Workbooks
        self.assertEqual(len(wbs), 0)

        wbs.Add()
        self.assertEqual(len(wbs), 1)

        wbs.Add()
        self.assertEqual(len(wbs), 2)

        for wb in wbs:
            self.assertIsNotNone(wb.Name)
            for sheet in wb.Sheets:
                self.assertIsNotNone(sheet.Name)

        app.Quit()


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3

import sys
import os

sys.path.insert(0, os.path.dirname(os.path.realpath(__file__)) + "/../build")

from pywpsrpc.rpcwpsapi import (createWpsRpcInstance, wpsapi)
from pywpsrpc.common import FAILED
from pywpsrpc import RpcProxy


def main():
    hr, rpc = createWpsRpcInstance()
    if FAILED(hr):
        print("createWpsRpcInstance failed with hr: ", hr)
        sys.exit(-1)

    app = RpcProxy(rpc.getWpsApplication())
    print(app.last_error)

    # get the raw object
    obj = app.rpc_object
    print(obj)

    print(app.Build)
    app.Visible = False
    app.Visible = True

    doc = app.Documents.Add()
    print(doc.Name)

    if not app.Selection.InsertAfter("Hello, world"):
        print("Can't insert text")

    font = app.Selection.Font
    font.Bold = True

    doc.Close(SaveChanges=wpsapi.wdDoNotSaveChanges)

    try:
        # should be failed since no Documents opened.
        doc = app.ActiveDocument
    except Exception as e:
        print(e)

    app.Quit(wpsapi.wdDoNotSaveChanges)


if __name__ == "__main__":
    main()



#!/usr/bin/python3

import sys
import os

sys.path.insert(0, os.path.dirname(os.path.realpath(__file__)) + "/../build")

from pywpsrpc.common import (FAILED, wpsapiex)
from pywpsrpc import RpcProxy


can_close_doc = False


def _onDocumentBeforeClose(doc):
    print("_onDocumentBeforeClose: ", doc.Name)
    return not can_close_doc


def _onDocumentBeforeSave(doc):
    print("__onDocumentBeforeSave: ", doc.Name)
    # SaveAsUI, Cancel
    return True, True


def _onDocumentChange():
    print("_onDocumentChange")


def _onDocumentOpen(doc):
    print("_onDocumentOpen:", doc.Name)


def _onNewDocument(doc):
    print("_onNewDocument:", doc.Name)


def _onQuit():
    print("_onQuit")


def test_rpcwpsapi():
    try:
        from pywpsrpc.rpcwpsapi import (createWpsRpcInstance, wpsapi)
    except ImportError:
        return
    hr, rpc = createWpsRpcInstance()
    if FAILED(hr):
        print("createWpsRpcInstance failed with hr: ", hr)
        sys.exit(-1)

    app = RpcProxy(rpc.getWpsApplication())

    hr = rpc.registerEvent(app.rpc_object,
                           wpsapi.DIID_ApplicationEvents4,
                           "DocumentBeforeClose",
                           _onDocumentBeforeClose)
    print("registerEvent:", hex(hr & 0xFFFFFFFF))

    hr = rpc.registerEvent(app.rpc_object,
                           wpsapi.DIID_ApplicationEvents4,
                           "DocumentBeforeSave",
                           _onDocumentBeforeSave)

    print("registerEvent:", hex(hr & 0xFFFFFFFF))

    hr = rpc.registerEvent(app.rpc_object,
                           wpsapi.DIID_ApplicationEvents4,
                           "DocumentChange",
                           _onDocumentChange)

    hr = rpc.registerEvent(app.rpc_object,
                           wpsapi.DIID_ApplicationEvents4,
                           "DocumentOpen",
                           _onDocumentOpen)

    hr = rpc.registerEvent(app.rpc_object,
                           wpsapi.DIID_ApplicationEvents4,
                           "NewDocument",
                           _onNewDocument)

    hr = rpc.registerEvent(app.rpc_object,
                           wpsapi.DIID_ApplicationEvents4,
                           "Quit",
                           _onQuit)

    def _onWindowActivate(doc, window):
        ss = {wpsapi.wdWindowStateNormal: "wdWindowStateNormal",
            wpsapi.wdWindowStateMaximize: "wdWindowStateMaximize",
            wpsapi.wdWindowStateMinimize: "wdWindowStateMinimize"}
        print("_onWindowActivate:", doc.Name, window.Caption, ss[window.WindowState])

    hr = rpc.registerEvent(app.rpc_object,
                           wpsapi.DIID_ApplicationEvents4,
                           "WindowActivate",
                           _onWindowActivate)

    def _onWindowDeactivate(doc, window):
        print("_onWindowDeactivate:", window.Caption)

    hr = rpc.registerEvent(app.rpc_object,
                           wpsapi.DIID_ApplicationEvents4,
                           "WindowDeactivate",
                           _onWindowDeactivate)

    def _onWindowSelectionChange(selection):
        print("_onWindowSelectionChange:", selection.Start,
              selection.End, selection.Text)

    hr = rpc.registerEvent(app.rpc_object,
                           wpsapi.DIID_ApplicationEvents4,
                           "WindowSelectionChange",
                           _onWindowSelectionChange)

    doc = app.Documents.Add()
    # the doc should not be saved
    doc.SaveAs2("test.doc")

    # DocumentChange
    doc2 = app.Documents.Add()

    doc3 = app.Documents.Open(os.path.realpath(__file__))

    selection = app.Selection
    # selection should changed
    selection.SetRange(0, 10)

    # the doc should not be closed
    doc.Close()

    # now make it works
    global can_close_doc
    can_close_doc = True

    app.Quit()


def _onPresentationSave(Pres):
    print("_onPresentationSave: ", Pres)


def _onPresentationClose(Pres):
    print("_onPresentationClose: ", Pres)


def _onAfterNewPresentation(Pres):
    print("_onAfterNewPresentation: ", Pres.Name)


def _onAfterPresentationOpen(Pres):
    print("_onAfterPresentationOpen: ", Pres.Name)


def _onNewPresentation(Pres):
    print("_onNewPresentation: ", Pres.Name)


def _onDocumentAfterPrint(Pres, PageEx):
    print("_onDocumentAfterPrint")


def test_rpcwppapi():
    try:
        from pywpsrpc.rpcwppapi import (createWppRpcInstance, wppapi)
    except ImportError:
        return

    hr, rpc = createWppRpcInstance()
    if FAILED(hr):
        print("createWppRpcInstance failed with hr: ", hr)
        sys.exit(-1)

    app = RpcProxy(rpc.getWppApplication())

    hr = rpc.registerEvent(app.rpc_object,
                           wppapi.IID_EApplication,
                           "PresentationSave",
                           _onPresentationSave)
    print("registerEvent:", hex(hr & 0xFFFFFFFF))

    hr = rpc.registerEvent(app.rpc_object,
                           wppapi.IID_EApplication,
                           "PresentationClose",
                           _onPresentationClose)

    print("registerEvent:", hex(hr & 0xFFFFFFFF))

    hr = rpc.registerEvent(app.rpc_object,
                           wppapi.IID_EApplication,
                           "AfterNewPresentation",
                           _onAfterNewPresentation)

    hr = rpc.registerEvent(app.rpc_object,
                           wppapi.IID_EApplication,
                           "AfterPresentationOpen",
                           _onAfterPresentationOpen)

    hr = rpc.registerEvent(app.rpc_object,
                           wppapi.IID_EApplication,
                           "NewPresentation",
                           _onNewPresentation)

    def _onWindowActivate(doc, window):
        ss = {wppapi.ppWindowNormal: "ppWindowNormal",
              wppapi.ppWindowMinimized: "ppWindowMinimized",
              wppapi.ppWindowMaximized: "ppWindowMaximized"}
        print("_onWindowActivate:", window.Caption, ss[window.WindowState])

    hr = rpc.registerEvent(app.rpc_object,
                           wppapi.IID_EApplication,
                           "WindowActivate",
                           _onWindowActivate)

    def _onWindowDeactivate(doc, window):
        print("_onWindowDeactivate:", window.Caption)

    hr = rpc.registerEvent(app.rpc_object,
                           wppapi.IID_EApplication,
                           "WindowDeactivate",
                           _onWindowDeactivate)

    appEx = app.ApplicationEx
    hr = rpc.registerEvent(appEx.rpc_object,
                           wpsapiex.DIID_ApplicationEventsEx,
                           "DocumentAfterPrint",
                           _onDocumentAfterPrint)

    pres = app.Presentations.Add(wppapi.msoTrue)
    pres.SaveAs("test.ppt")
    pres.Close()

    pres = app.Presentations.Open(
        "test.ppt", wppapi.msoFalse,
        wppapi.msoFalse, wppapi.msoTrue)

    pres.PrintOut()

    app.Quit()


can_close_wb = False


def _onWorkbookBeforeClose(wb):
    print("_onWorkbookBeforeClose: ", wb.Name)
    return not can_close_wb


def _onWorkbookBeforeSave(wb):
    print("_onWorkbookBeforeSave: ", wb.Name)
    # SaveAsUI, Cancel
    return True, not can_close_wb


def _onWorkbookAfterSave(wb, success):
    print("_onWorkbookAfterSave: ", wb.Name, success)


def _onNewWorkbook(wb):
    print("_onNewWorkbook: ", wb.Name)


def _onWorkbookOpen(wb):
    print("_onWorkbookOpen: ", wb.Name)


def test_rpcetapi():
    try:
        from pywpsrpc.rpcetapi import (createEtRpcInstance, etapi)
    except ImportError:
        return

    hr, rpc = createEtRpcInstance()
    if FAILED(hr):
        print("createEtRpcInstance failed with hr: ", hr)
        sys.exit(-1)

    app = RpcProxy(rpc.getEtApplication())

    hr = rpc.registerEvent(app.rpc_object,
                           etapi.DIID_AppEvents,
                           "WorkbookBeforeClose",
                           _onWorkbookBeforeClose)
    print("registerEvent:", hex(hr & 0xFFFFFFFF))

    hr = rpc.registerEvent(app.rpc_object,
                           etapi.DIID_AppEvents,
                           "WorkbookBeforeSave",
                           _onWorkbookBeforeSave)

    print("registerEvent:", hex(hr & 0xFFFFFFFF))

    hr = rpc.registerEvent(app.rpc_object,
                           etapi.DIID_AppEvents,
                           "WorkbookAfterSave",
                           _onWorkbookAfterSave)

    hr = rpc.registerEvent(app.rpc_object,
                           etapi.DIID_AppEvents,
                           "NewWorkbook",
                           _onNewWorkbook)

    hr = rpc.registerEvent(app.rpc_object,
                           etapi.DIID_AppEvents,
                           "WorkbookOpen",
                           _onWorkbookOpen)

    def _onWindowActivate(wb, window):
        ss = {etapi.xlMaximized: "xlMaximized",
              etapi.xlMinimized: "xlMinimized",
              etapi.xlNormal: "xlNormal"}
        print("_onWindowActivate:", window.Caption, ss[window.WindowState])

    hr = rpc.registerEvent(app.rpc_object,
                           etapi.DIID_AppEvents,
                           "WindowActivate",
                           _onWindowActivate)

    def _onWindowDeactivate(wb, window):
        print("_onWindowDeactivate:", wb.Name, window.Caption)

    hr = rpc.registerEvent(app.rpc_object,
                           etapi.DIID_AppEvents,
                           "WindowDeactivate",
                           _onWindowDeactivate)

    wb = app.Workbooks.Add()
    # the doc should not be saved
    wb.SaveAs("test.xls")
    # the doc should not be closed
    wb.Close()

    # now make it works
    global can_close_wb
    can_close_wb = True

    # save again
    wb.SaveAs("test2.xls")
    wb.Close()

    wb = app.Workbooks.Open("test2.xls")

    app.Quit()


if __name__ == "__main__":
    test_rpcwpsapi()
    test_rpcwppapi()
    test_rpcetapi()



#**
# * Copyright (c) 2020 Weitian Leung
# *
# * This file is part of pywpsrpc.
# *
# * This file is distributed under the MIT License.
# * See the LICENSE file for details.
# *
#*


from .common import (S_OK, IUnknown)
from types import BuiltinFunctionType


class RpcException(Exception):

    def __init__(self, text, hr=None):
        self._text = text
        self._hr = hr

    def __str__(self):
        text = "RpcException: {}".format(self._text)
        if self._hr is not None:
            text += " ({})".format(hex(self._hr & 0xffffffff))

        return text


class RpcMethod(object):
    __slots__ = ["_method", "_proxy"]

    def __init__(self, method, proxy):
        self._proxy = None
        if not isinstance(method, BuiltinFunctionType):
            raise RpcException("RpcMethod required builtin function or method")

        self._method = method
        self._proxy = proxy

    def __call__(self, *args, **kwargs):
        ret = self._method(*args, **kwargs)
        if isinstance(ret, tuple):
            self._proxy.last_error = ret[0]
            if ret[0] != S_OK:
                if self._proxy.use_exception:
                    raise RpcException("Call '{}' failed.".format(
                        self._method), ret[0])
                return None

            if len(ret) == 2:
                if isinstance(ret[1], IUnknown):
                    return RpcProxy(ret[1], self._proxy.use_exception)
                else:
                    return ret[1]

            result = ()
            for i in range(1, len(ret)):
                if isinstance(ret[i], IUnknown):
                    result += (RpcProxy(ret[i], self._proxy.use_exception), )
                else:
                    result += (ret[i], )
            return result

        self._proxy.last_error = ret
        return ret == S_OK

    @property
    def __doc__(self):
        return self._method.__doc__


class RpcIter(object):
    def __init__(self, rpc_object):
        self._obj = rpc_object
        # starts from 1 not 0
        self._index = 1
        self._count = len(rpc_object)

    def __iter__(self):
        return self

    def __next__(self):
        if self._index > self._count:
            raise StopIteration

        value = self._obj[self._index]
        self._index += 1

        return value


class RpcProxy(object):
    """ Deprecated
    use RpcProxy only if you want to deduce the HRESULT
    """

    __slots__ = ["_object", "_use_exception", "_last_hr"]

    def __init__(self, obj, use_exception=False):
        """ The obj can be (hr, IUnknown) or IUnknown.
        If use_exception set to True then any call failed will raise an exception.
        """

        def _check_iunknown(obj):
            if not isinstance(obj, IUnknown):
                raise RpcException("RpcProxy required an IUnknown instance")

        self._object = None
        self._last_hr = S_OK
        if isinstance(obj, tuple):
            self._last_hr = obj[0]
            if obj[0] == S_OK:
                _check_iunknown(obj[1])
                self._object = obj[1]
            elif use_exception:
                raise RpcException(
                    "Can't create proxy due to the previous call failed.", obj[0])
        else:
            _check_iunknown(obj)
            self._object = obj

        self._use_exception = use_exception

    def __getattr__(self, name):
        value = getattr(self._object, name)
        if isinstance(value, BuiltinFunctionType):
            return RpcMethod(value, self)
        elif isinstance(value, IUnknown):
            value = RpcProxy(value, self._use_exception)

        self._last_hr = S_OK
        return value

    def __setattr__(self, name, value):
        if name in self.__slots__ or name in ("use_exception", "last_error", "rpc_object"):
            super().__setattr__(name, value)
        else:
            setattr(self._object, name, value)
            self._last_hr = S_OK

    def __bool__(self):
        return self._object is not None

    def __len__(self):
        return len(self._object)

    def __getitem__(self, index):
        item = self._object.__getitem__(index)
        if isinstance(item, IUnknown):
            item = RpcProxy(item, self._use_exception)

        return item

    def __iter__(self):
        if hasattr(self._object, "__iter__"):
            return iter(self._object)

        return RpcIter(self)

    @property
    def rpc_object(self):
        return self._object

    @property
    def use_exception(self):
        return self._use_exception

    @use_exception.setter
    def use_exception(self, value):
        self._use_exception = value

    @property
    def last_error(self):
        return self._last_hr

    @last_error.setter
    def last_error(self, hr):
        self._last_hr = hr



#!/usr/bin/env python3

import sys
import os
import datetime
import unittest

sys.path.insert(0, os.path.dirname(os.path.realpath(__file__)) + "/../build")

from pywpsrpc.rpcwppapi import wppapi
from pywpsrpc import (rpcwppapi, common)
from pywpsrpc.utils import RpcIter

class TestRpcWppApi(unittest.TestCase):

    def check_call(self, funcName, hr, value=None):
        self.assertEqual(hr, common.S_OK, funcName)

    def test(self):
        hr, rpc = rpcwppapi.createWppRpcInstance()
        self.check_call("rpcwppapi.createWppRpcInstance", hr, rpc)

        hr, app = rpc.getWppApplication()
        self.check_call("rpc.getWppApplication", hr, app)

        hr, pid = rpc.getProcessPid()
        self.check_call("rpc.getProcessPid", hr, pid)

        hr, preses = app.get_Presentations()
        self.check_call("app.get_Presentations", hr, preses)

        hr, pres = preses.Add(wppapi.msoTrue)
        self.check_call("preses.Add", hr, pres)

        hr, pres = app.get_ActivePresentation()
        self.check_call("app.get_ActivePresentation", hr, pres)

        hr, name = pres.get_FullName()
        self.check_call("pres.get_FullName", hr, name)

        props = pres.CustomDocumentProperties
        for prop in RpcIter(props):
            self.assertIsNotNone(prop.Name)

        a_date = datetime.date(2020, 7, 20)
        hr, prop = props.Add("_a_date", False, Value=a_date)
        self.assertTrue(hr == common.S_OK)
        self.assertEqual(prop.Value.date(), a_date)

        a_datetime = datetime.datetime(2020, 7, 21, 14, 15, 30)
        hr, prop = props.Add("_a_datetime", False, Value=a_datetime)
        self.assertTrue(hr == common.S_OK)
        self.assertEqual(prop.Value, a_datetime)

        props = pres.BuiltInDocumentProperties
        for prop in RpcIter(props):
            self.assertIsNotNone(prop.Name)

        hr = pres.Close()
        self.check_call("pres.Close", hr)

        hr = app.Quit()
        self.check_call("ap.Quit", hr)


if __name__ == "__main__":
    unittest.main()



# **
# * Copyright (c) 2020-2023 Weitian Leung
# *
# * This file is part of pywpsrpc.
# *
# * This file is distributed under the MIT License.
# * See the LICENSE file for details.
# *
# *

import sipbuild
import os
import sys
import shutil
import re
import platform


var_pattern = re.compile(
    rb"(\s+::([a-zA-Z0-9_]+::)?[a-zA-Z0-9_]+\* a[0-9]+);$")


class PyWpsRpcProject(sipbuild.Project):
    def __init__(self):
        super().__init__(dunder_init=True)
        self.abi_version = "12"

        sdk_dir = os.path.join(self.root_dir, "wpsrpc-sdk")

        self.bindings_factories = [RpcCommon, RpcWpsApi, RpcWppApi, RpcEtApi]
        self.sdk_inc_dir = os.path.join(sdk_dir, "include")
        self.cxx_inc_dir = os.path.join(self.root_dir, "include")

        if not os.path.exists(self.sdk_inc_dir) or not os.path.isdir(self.sdk_inc_dir):
            raise sipbuild.UserException(
                "Missing sdk, please checkout the 'wpsrpc-sdk' submodule and try again")

        # always try system's one first
        dirs = ["/opt/kingsoft/wps-office/office6",
                "/usr/lib/office6",  # for Arch Linux
                "/opt/apps/cn.wps.wps-office/files/kingsoft/wps-office/office6",  # Deepin
                sdk_dir + "/lib/" + platform.machine()
                ]

        self.sdk_lib_dir = None
        for dir in dirs:
            if os.path.exists(dir) and os.path.isdir(dir):
                self.sdk_lib_dir = dir
                break

    def apply_nonuser_defaults(self, tool):
        if self.builder_factory is None:
            self.builder_factory = RpcApiBuilder

        if self.sip_files_dir is None:
            self.sip_files_dir = "sip"

        # must set sip_module
        if self.sip_module is None:
            self.sip_module = "pywpsrpc.sip"

        if self.sip_include_dirs is None:
            self.sip_include_dirs = [self.root_dir + "/sip/common"]

        super().apply_nonuser_defaults(tool)

    def get_dunder_init(self):
        dunder_init = ""
        with open(os.path.join(self.root_dir, "py", "__init__.py")) as f:
            dunder_init = f.read()

        return dunder_init

    def get_requires_dists(self):
        # the sip module already contains in the package
        return []


class RpcApiBindings(sipbuild.Bindings):

    def __init__(self, project, name, **kwargs):
        super().__init__(project, name, **kwargs)
        self.pep484_pyi = True

    def get_options(self):
        options = super().get_options()

        options.append(sipbuild.Option("QT", option_type=list))
        options.append(sipbuild.Option("PCH", option_type=list))

        return options

    def apply_nonuser_defaults(self, tool):
        if self.sip_file is None:
            self.sip_file = self.name + "/" + self.name + ".sip"

        super().apply_nonuser_defaults(tool)

        self.builder_settings.append("QT = %s" % ' '.join(self.QT))

    def apply_user_defaults(self, tool):
        super().apply_user_defaults(tool)

    def generate(self):
        buildable = super().generate()

        pch_file = os.path.join(buildable.build_dir, "stdafx.h")
        with open(pch_file, "w+") as f:
            f.write("#ifndef PYWPSRPC_%s_H\n" % buildable.target)
            f.write("#define PYWPSRPC_%s_H\n\n" % buildable.target)
            for pch in self.PCH:
                f.write("#include <%s>\n" % pch)

            f.write("#include <wpsapiex.h>\n")
            f.write("#include <string>\n")
            f.write("#include <vector>\n\n")
            f.write("#endif\n")

        buildable.headers.append(pch_file)

        # asyncio seems no improves here
        self._fix_uninited_vars(buildable.sources)

        return buildable

    def _fix_uninited_vars(self, sources):
        """ Try init the pointer argument
        We need it because of using sipEventCollectingWrapper
        """
        # FIXME: this should be done by SIP
        for src in sources:
            tmp_file = src + "_tmp"
            f = open(src, "rb")
            f_tmp = open(tmp_file, "wb+")

            for line in f:
                m = var_pattern.match(line)
                if m:
                    line = m.group(1) + b' = nullptr;\n'
                f_tmp.write(line)

            f.close()
            f_tmp.close()
            os.rename(tmp_file, src)


class RpcCommon(RpcApiBindings):

    def __init__(self, project):
        dirs = [project.sdk_inc_dir, project.sdk_inc_dir + "/common"]
        super().__init__(project,
                         "common",
                         include_dirs=dirs,
                         QT=["core"],
                         PCH=["pre_stddef.h",
                              "kfc/guid.h",
                              "int.h",
                              "typedef.h",
                              "guiddef.h",
                              "objbase.h",
                              "strapi/strapi.h",
                              "comdef.h",
                              "ksoapi/ksoapi.h"])

    def generate(self):
        buildable = super().generate()
        self._fix_sip(buildable)
        return buildable

    def _fix_sip(self, buildable):
        for installable in buildable.installables:
            if installable.name != "sip":
                continue

            self._fix_subdir_sip(buildable, installable)

            # The export.sip include in other module
            common_dir = os.path.join(self.project.root_dir, "sip", "common")
            installable.files.append(common_dir + "/export.sip")

            break

    def _fix_subdir_sip(self, buildable, sip_installable):
        subdirs = ["ksoapi", "wpsapiex"]
        for subdir in subdirs:
            sub_installable = sipbuild.Installable(
                "sip_" + subdir,
                target_subdir=sip_installable.target_subdir + "/" + subdir)

            sub_path = "/" + subdir + "/"
            sub_installable.files = [
                f for f in sip_installable.files if sub_path in f]

            sip_installable.files = [f for f in sip_installable.files
                                     if f not in sub_installable.files]
            buildable.installables.append(sub_installable)


class RpcWpsApi(RpcApiBindings):

    def __init__(self, project):
        dirs = [project.sdk_inc_dir,
                project.sdk_inc_dir + "/common",
                project.sdk_inc_dir + "/wps",
                project.cxx_inc_dir]

        super().__init__(project,
                         "rpcwpsapi",
                         include_dirs=dirs,
                         libraries=["rpcwpsapi_sysqt5"],
                         library_dirs=[project.sdk_lib_dir],
                         QT=["core"],
                         PCH=["QString", "wps/wpsapi.h"])


class RpcWppApi(RpcApiBindings):

    def __init__(self, project):
        dirs = [project.sdk_inc_dir,
                project.sdk_inc_dir + "/common",
                project.sdk_inc_dir + "/wpp",
                project.cxx_inc_dir]

        super().__init__(project,
                         "rpcwppapi",
                         include_dirs=dirs,
                         libraries=["rpcwppapi_sysqt5"],
                         library_dirs=[project.sdk_lib_dir],
                         PCH=["wpp/wppapi.h"])


class RpcEtApi(RpcApiBindings):

    def __init__(self, project):
        dirs = [project.sdk_inc_dir,
                project.sdk_inc_dir + "/common",
                project.sdk_inc_dir + "/et",
                project.cxx_inc_dir]

        super().__init__(project,
                         "rpcetapi",
                         include_dirs=dirs,
                         libraries=["rpcetapi_sysqt5"],
                         library_dirs=[project.sdk_lib_dir],
                         PCH=["et/etapi.h"])


# seems that we have no easy way to change the cxx flags
# we need to set the RPATH flags at least
class RpcApiBuilder(sipbuild.Builder):
    def __init__(self, project, **kwargs):
        super().__init__(project, **kwargs)

    def get_options(self):
        options = super().get_options()

        options.append(
            sipbuild.Option("qmake",
                            metavar="<FILE>",
                            help="Specify the qmake executable"))
        options.append(
            sipbuild.Option("clang",
                            option_type=bool,
                            help="Use clang to compile the project"))

        options.append(
            sipbuild.Option("libcxx",
                            option_type=bool,
                            help="Link libc++ when --clang specified"))
        return options

    def apply_user_defaults(self, tool):
        if tool in sipbuild.Option.BUILD_TOOLS:
            if self.qmake is None:
                self.qmake = shutil.which("qmake-qt5")
                if not self.qmake:
                    self.qmake = shutil.which("qmake")

            if not self.qmake:
                raise sipbuild.UserException(
                    "Please specify a valid qmake executable")

        super().apply_user_defaults(tool)

    def build_executable(self, buildable, *, fatal=True):
        raise sipbuild.UserException("RpcApiBuilder cannot build executables")

    def build_project(self, target_dir, *, wheel_tag=None):
        installed = []
        sub_dirs = []

        is_debug = False
        for buildable in self.project.buildables:
            if isinstance(buildable, sipbuild.BuildableModule):
                is_debug = buildable.debug
                self._gen_module_pro_file(buildable, target_dir, installed)
            elif type(buildable) is sipbuild.Buildable:
                for installable in buildable.installables:
                    installable.install(
                        target_dir, installed, do_install=False)
            else:
                raise sipbuild.UserException(
                    "RpcApiBuilder cannot build '{0}' buildables".format(
                        type(buildable).__name__))

            sub_dirs.append(buildable.name)

        self._gen_sip_project(target_dir, sub_dirs, installed, is_debug)

        self.project.progress("Generating the top-level project")

        pro = self.project.build_dir + "/" + self.project.name + ".pro"
        with open(pro, "w+") as f:
            f.write("TEMPLATE = subdirs\n")
            f.write("CONFIG += ordered\n\n")
            f.write("SUBDIRS = %s\n\n" % ' '.join(sub_dirs))

            for installable in self.project.installables:
                self._install(f, installable, target_dir, installed)

            py_subdir = os.path.join(target_dir, self.project.name)
            sip_py = sipbuild.Installable("sip_py", target_subdir=py_subdir)
            py_dir = os.path.join(self.project.root_dir, "py")
            sip_py_files = [(py_dir + "/" + f) for f in os.listdir(py_dir)
                            if f.endswith(".py") and f != "__init__.py"]
            sip_py.files.extend(sip_py_files)
            self._install(f, sip_py, target_dir, installed)

            fake_root = os.path.join(self.project.build_dir, self.project.name)
            for py in sip_py.files:
                shutil.copy(py, fake_root)
            shutil.copy(os.path.join(py_dir, "__init__.py"), fake_root)

            # for distinfo
            inventory_file = self.project.build_dir + "/inventory.txt"
            with open(inventory_file, "w+") as inventory:
                inventory.write('\n'.join(installed))
                inventory.write('\n')

            distinfo = ["sip-distinfo",
                        "--project-root", self.project.root_dir,
                        "--generator", os.path.basename(sys.argv[0]),
                        "--prefix", '\\"$(INSTALL_ROOT)\\"',
                        "--inventory", inventory_file]

            if wheel_tag:
                distinfo.append("--wheel-tag")
                distinfo.append(wheel_tag)

            distinfo.append(self.project.get_distinfo_dir(target_dir))

            f.write("distinfo.extra = %s\n" % ' '.join(distinfo))
            f.write("distinfo.path = %s\n" % target_dir)
            f.write("INSTALLS += distinfo\n")

        old_dir = os.getcwd()
        os.chdir(self.project.build_dir)

        args = [self.qmake, "-recursive", self.project.name + ".pro"]
        if self.clang:
            if self.libcxx:
                args.extend(["-spec", "linux-clang-libc++"])
            else:
                args.extend(["-spec", "linux-clang"])
        self.project.run_command(args, fatal=True)

        self.project.progress("Compiling the project")
        self._run_make()
        os.chdir(old_dir)

    def install_project(self, target_dir, *, wheel_tag=None):
        self.project.progress("Installing the project")
        old_dir = os.getcwd()
        os.chdir(self.project.build_dir)
        self._run_make(True)
        os.chdir(old_dir)

    def _gen_module_pro_file(self, buildable, target_dir, installed):
        self.project.progress("Generating the %s project" % buildable.target)

        with open(buildable.build_dir + "/" + buildable.target + ".pro", "w+") as f:
            f.write("TEMPLATE = lib\n")
            f.write("CONFIG += plugin no_plugin_name_prefix warn_off\n")
            f.write("CONFIG += %s\n" %
                    ("debug" if buildable.debug else "release"))
            f.write("CONFIG += c++11 precompile_header\n")
            f.write("%s\n" % ' '.join(buildable.builder_settings))
            f.write("TARGET = %s\n\n" % buildable.target)

            if buildable.define_macros:
                f.write("DEFINES += %s\n" % ' '.join(buildable.define_macros))

            f.write("INCLUDEPATH += .\n")
            for dir in buildable.include_dirs:
                f.write("INCLUDEPATH += %s\n" % dir)
            f.write("INCLUDEPATH += %s\n\n" % self.project.py_include_dir)

            for dir in buildable.library_dirs:
                f.write("LIBS += -L%s\n" % dir)
            for lib in buildable.libraries:
                f.write("LIBS += -l%s\n" % lib)

            f.write('\n')

            # hidden symbols
            with open(buildable.build_dir + "/" + buildable.target + ".exp", "w+") as exp:
                exp.write("{ global: PyInit_%s; local: *; };" %
                          buildable.target)

            f.write("QMAKE_LFLAGS += -Wl,--version-script=%s.exp\n" %
                    buildable.target)
            if not buildable.debug:
                f.write("QMAKE_LFLAGS += -s\n")
            # f.write("QMAKE_LFLAGS_PLUGIN += -Wl,--no-undefined\n")
            f.write(
                "QMAKE_CXXFLAGS += -Wno-attributes -Wno-delete-non-virtual-dtor\n")
            f.write("QMAKE_CXXFLAGS += -Wno-delete-incomplete -Wno-unused-variable\n")
            f.write("QMAKE_RPATHDIR += $ORIGIN /opt/kingsoft/wps-office/office6\n")
            f.write("QMAKE_RPATHDIR += /usr/lib/office6\n")
            f.write("QMAKE_RPATHDIR += /opt/apps/cn.wps.wps-office/files/kingsoft/wps-office/office6\n\n")

            # for testing
            rpc_dir = os.path.join(self.project.build_dir, self.project.name)
            pyi = os.path.join(buildable.build_dir, buildable.target + ".pyi")
            os.makedirs(rpc_dir, exist_ok=True)
            shutil.copy(pyi, rpc_dir)

            f.write("QMAKE_POST_LINK = $(COPY_FILE) $(TARGET) %s\n\n" % rpc_dir)
            f.write("PRECOMPILED_HEADER = stdafx.h\n\n")

            f.write("HEADERS = %s\n" % " \\\n\t".join(buildable.headers))
            f.write("SOURCES = %s" % " \\\n\t".join(buildable.sources))
            f.write('\n\n')

            f.write("target.path = %s\n" %
                    (target_dir + "/" + self.project.name))
            f.write("INSTALLS += target\n\n")

            # for dist-info
            target_installable = sipbuild.Installable(
                "target", target_subdir=buildable.get_install_subdir())
            installed.append(
                target_installable.get_full_target_dir(target_dir) +
                "/%s.so" % buildable.target)

            for installable in buildable.installables:
                self._install(f, installable, target_dir, installed)

    def _install(self, f, installable, target_dir, installed):
        installable.install(target_dir, installed, do_install=False)

        f.write("{}.path = {}\n".format(
            installable.name,
            installable.get_full_target_dir(target_dir)))
        f.write("{}.files = {}\n".format(
            installable.name,
            " \\\n\t".join(installable.files)))
        f.write("INSTALLS += %s\n\n" % installable.name)

    def _run_make(self, install=False):
        if install:
            # single thread only
            args = ["make", "install"]
        else:
            args = ["make", "-j%s" % os.cpu_count()]

        self.project.run_command(args, fatal=True)

    def _gen_sip_project(self, target_dir, sub_dirs, installed, is_debug):
        self.project.progress("Generating the sip project")

        sub_dir = self.project.build_dir + "/sip"
        os.makedirs(sub_dir, exist_ok=True)

        if sipbuild.SIP_VERSION >= 0x060200:
            main_ver = self.project.abi_version.split('.')[0]
        else:
            main_ver = self.project.abi_version
        sources = sipbuild.module.copy_nonshared_sources(
            main_ver, sub_dir)

        # use copy_sip_h instead?
        shutil.copy(self.project.build_dir + "/sip.h", sub_dir)

        with open(sub_dir + "/sip.pro", "w+") as f:
            f.write("TEMPLATE = lib\n")

            f.write("CONFIG += plugin no_plugin_name_prefix warn_on\n")
            f.write("CONFIG += %s\n" % ("debug" if is_debug else "release"))

            f.write("QT =\n")
            f.write("TARGET = sip\n\n")

            f.write("INCLUDEPATH += %s\n\n" % self.project.py_include_dir)

            with open(sub_dir + "/sip.exp", "w+") as exp:
                exp.write("{ global: PyInit_sip; local: *; };")

            f.write("QMAKE_LFLAGS += -Wl,--version-script=sip.exp\n")
            if not is_debug:
                f.write("QMAKE_LFLAGS += -s\n")
            f.write("\n")

            rpc_dir = os.path.join(self.project.build_dir, self.project.name)
            os.makedirs(rpc_dir, exist_ok=True)
            f.write("QMAKE_POST_LINK = $(COPY_FILE) $(TARGET) %s\n\n" % rpc_dir)

            f.write("SOURCES = %s\n\n" % " \\\n\t".join(sources))

            f.write("target.path = %s\n" %
                    (target_dir + "/" + self.project.name))
            f.write("INSTALLS += target\n\n")

        sub_dirs.append("sip")
        installed.append(target_dir + "/" + self.project.name + "/sip.so")



#!/usr/bin/env python3

import sys
import os
import unittest

sys.path.insert(0, os.path.dirname(os.path.realpath(__file__)) + "/../build")
from pywpsrpc.rpcwpsapi import createWpsRpcInstance
from pywpsrpc.rpcetapi import createEtRpcInstance

class TestWps(unittest.TestCase):

    def test_wps(self):
        hr, rpc = createWpsRpcInstance()
        hr, app = rpc.getWpsApplication()

        visible = app.Visible
        self.assertTrue(visible)

        app.Visible = False
        self.assertFalse(app.Visible)

        app.Visible = True

        self.assertEqual(app.Documents.Count, 0)

        try:
            doc = app.ActiveDocument
            self.assertTrue(False, "should not go here")
        except AttributeError as e:
            self.assertTrue(True)

        try:
            app.ActiveDocument = None
            self.assertTrue(False, "should not go here")
        except AttributeError as e:
            self.assertTrue(True)

        hr, doc = app.Documents.Add()
        self.assertEqual(app.Documents.Count, 1)

        self.assertIsNotNone(doc.Name)
        doc.Close()

        app.Quit()


class Testet(unittest.TestCase):

    def test_et(self):
        hr, rpc = createEtRpcInstance()
        hr, app = rpc.getEtApplication()

        workbooks = app.Workbooks
        self.assertEqual(workbooks.Count, 0)

        hr, workbook = workbooks.Add()
        self.assertEqual(workbooks.Count, 1)

        sheet = workbook.ActiveSheet
        self.assertIsNotNone(sheet.Name)

        hr, rg = sheet.get_Range("A1")
        rg.Value = "test"

        self.assertEqual(sheet.get_Range("A1")[1].Value, "test")

        workbook.Close(SaveChanges=False)

        app.Quit()


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/python3
# -- coding: utf-8 -
# **
# * Copyright (c) 2022 cong.zheng
# *
# * This file is part of pywpsrpc.
# *
# * This file is distributed under the MIT License.
# * See the LICENSE file for details.
# *
# *

import os
import subprocess
import sys

import argparse

from pywpsrpc.rpcetapi import (createEtRpcInstance, etapi)

from pywpsrpc.common import (S_OK, QtApp)

formats = {
    "pdf": etapi.XlFixedFormatType.xlTypePDF,
}

pid = None


class ConvertException(Exception):

    def __init__(self, text, hr):
        self.text = text
        self.hr = hr

    def __str__(self):
        return """Convert failed:
Details: {}
ErrCode: {}
""".format(self.text, hex(self.hr & 0xFFFFFFFF))


def convert_to(paths, format, abort_on_fails=False):
    hr, rpc = createEtRpcInstance()
    if hr != S_OK:
        raise ConvertException("Can't create the rpc instance", hr)

    hr, app = rpc.getEtApplication()
    if hr != S_OK:
        raise ConvertException("Can't get the application", hr)
    global pid
    hr, pid = rpc.getProcessPid()
    if hr != S_OK:
        raise ConvertException("Can't  get the PID", hr)
    print("PID:{}".format(pid))
    # we don't need the gui
    app.Visible = False

    docs = app.Workbooks

    def _handle_result(hr):
        if abort_on_fails and hr != S_OK:
            raise ConvertException("convert_file failed", hr)

    for path in paths:
        abs_path = os.path.realpath(path)
        if os.path.isdir(abs_path):
            files = [(os.path.join(abs_path, f)) for f in os.listdir(abs_path)]
            for file in files:
                hr = convert_file(file, docs, format)
                _handle_result(hr)
        else:
            hr = convert_file(abs_path, docs, format)
            _handle_result(hr)

    app.Quit()


def convert_file(file, docs, format):
    hr, doc = docs.Open(file, Password='xxx', ReadOnly=True)
    if hr != S_OK:
        return hr

    out_dir = os.path.dirname(os.path.realpath(file)) + "/out"
    os.makedirs(out_dir, exist_ok=True)

    # you have to handle if the new_file already exists
    new_file = out_dir + "/" + os.path.splitext(os.path.basename(file))[0] + "." + format
    ret = doc.ExportAsFixedFormat(formats[format], new_file)

    # always close the doc
    doc.Close()

    return ret


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("--format", "-f",
                        required=True,
                        metavar="<DOC_TYPE>",
                        choices=["pdf"],
                        help="convert to <DOC_TYPE>,")

    parser.add_argument("--abort", "-a",
                        action="store_true",
                        help="abort if one convert fails")

    parser.add_argument("path",
                        metavar="<path>",
                        nargs='+',
                        help="the <path> can be one or more file or folder")

    args = parser.parse_args()

    qApp = QtApp(sys.argv)
    try:
        convert_to(args.path, args.format, args.abort)
        print("conversion completed")
    except Exception as e:
        print(e)
    finally:
        if pid is not None:
            print("kill et")
            subprocess.call("kill -9 {}".format(pid), shell=True)


if __name__ == "__main__":
    main()
